/* Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.model.sinfo;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.dive4elements.river.backend.SessionHolder;
import org.dive4elements.river.model.River;
import org.hibernate.Query;
import org.hibernate.Session;

/**
 * Hibernate binding for the DB table tkh
 *
 * @author Matthias Schäfer
 *
 */

@Entity
@Table(name = "tkh")
public class Tkh implements Serializable {

    /**
     * Possible values of the groupname column
     *
     */
    public enum Group {
        NONE(""), CALCULATION("berechnung"), MEASUREMENT("messung");

        private final String dbvalue;

        Group(final String dbvalue) {
            this.dbvalue = dbvalue;
        }

        /**
         * Value of the group's groupname column
         */
        public String getDbValue() {
            return this.dbvalue;
        }

        /**
         * Group for a groupname column value
         */
        public static Group forDbValue(final String dbvalue) {
            for (final Group g : Group.values()) {
                if (g.getDbValue().equals(dbvalue))
                    return g;
            }
            return Group.NONE;
        }
    }

    /***** FIELDS *****/

    private static final long serialVersionUID = 5891445651697221261L;

    private Integer id;

    private River river;

    private String filename;

    private transient Group group;

    private String kmrange_info;

    private String notes;

    private Date day;

    private String sounding_info;

    private String evaluation_by;

    private List<TkhColumn> columns;


    /***** CONSTRUCTORS *****/

    public Tkh() {
    }


    public Tkh(final River river, final String filename, final String kmrange_info, final String notes, final Date day, final String sounding_info,
            final String evaluation_by, final Group group) {
        this.river = river;
        this.filename = filename;
        this.kmrange_info = kmrange_info;
        this.notes = notes;
        this.day = day;
        this.setSounding_info(sounding_info);
        this.setEvaluation_by(evaluation_by);
        this.setGroup(group);
        this.columns = new ArrayList<>();
    }

    /***** METHODS *****/

    @Id
    @SequenceGenerator(name = "SEQUENCE_TKH_ID_SEQ", sequenceName = "TKH_ID_SEQ", allocationSize = 1)
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQUENCE_TKH_ID_SEQ")
    @Column(name = "id")
    public Integer getId() {
        return this.id;
    }

    public void setId(final Integer id) {
        this.id = id;
    }

    @OneToOne
    @JoinColumn(name = "river_id")
    public River getRiver() {
        return this.river;
    }

    public void setRiver(final River river) {
        this.river = river;
    }

    @Column(name = "filename")
    public String getFilename() {
        return this.filename;
    }

    public void setFilename(final String filename) {
        this.filename = filename;
    }

    @Column(name = "groupname")
    public String getGroupname() {
        return this.group.getDbValue();
    }

    public void setGroupname(final String groupname) {
        this.group = Group.forDbValue(groupname);
    }

    @Transient
    public Group getGroup() {
        return this.group;
    }

    public void setGroup(final Group group) {
        this.group = group;
    }

    @Column(name = "kmrange_info")
    public String getKmrange_info() {
        return this.kmrange_info;
    }

    public void setKmrange_info(final String kmrange_info) {
        this.kmrange_info = kmrange_info;
    }

    @Column(name = "notes")
    public String getNotes() {
        return this.notes;
    }

    public void setNotes(final String notes) {
        this.notes = notes;
    }

    @Column(name = "day")
    public Date getDay() {
        return this.day;
    }

    public void setDay(final Date day) {
        this.day = day;
    }

    @Column(name = "sounding_info")
    public String getSounding_info() {
        return this.sounding_info;
    }

    public void setSounding_info(final String sounding_info) {
        this.sounding_info = sounding_info;
    }

    @Column(name = "evaluation_by")
    public String getEvaluation_by() {
        return this.evaluation_by;
    }

    public void setEvaluation_by(final String evaluation_by) {
        this.evaluation_by = evaluation_by;
    }

    @OneToMany
    @JoinColumn(name = "tkh_id")
    public List<TkhColumn> getColumns() {
        return this.columns;
    }

    public void setColumns(final List<TkhColumn> values) {
        this.columns = values;
    }

    public void addColumn(final TkhColumn value) {
        this.columns.add(value);
    }

    /**
     * Selects a tkh series by id from the database
     */
    public static Tkh getTkhById(final int id) {
        final Session session = SessionHolder.HOLDER.get();
        final Query query = session.createQuery("FROM Tkh WHERE id=:id");
        query.setParameter("id", id);
        final List<Tkh> rows = query.list();
        return (rows != null) ? rows.get(0) : null;
    }
}